Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@babel/core");
const core_2 = require("@webpackery/core");
const lodash_1 = require("lodash");
const tsconfig_paths_webpack_plugin_1 = require("tsconfig-paths-webpack-plugin");
const inject_empty_exports_1 = require("./inject-empty-exports");
const ts_utils_1 = require("./ts-utils");
const omitKeys = [
    "extensions",
    "tsconfigPath",
    "useReact",
    "useTypesScript",
];
class BabelConfigurer extends core_2.WebpackConfigurer {

    prepare(props, context) {
        const { production } = context;

        props = Object.assign({}, props);
        props.options = Object.assign({ sourceMaps: production, envName: production ? "production" : "development" }, props.options);
        const partial = core_1.loadPartialConfig(Object.assign({ filename: __filename }, props.options));
        const loaded = core_1.loadOptions(Object.assign({ filename: __filename }, props.options));

        core_2.setDefaultBy(props, "useReact", isReact, partial, loaded.plugins);
        core_2.setDefaultBy(props, "useTypesScript", isTypeScript, partial, loaded.plugins);
        core_2.setDefaultBy(props, "extensions", resolveDefaultExtensions, props);
        core_2.setDefaultBy(props, "test", extensionsToRegex, props.extensions);
        return props;
    }

    configure(context, props) {
        const custom = {
            plugins: [],
        };

        if (props.useTypesScript) {
            const options = {
                preserveConstEnums: false,
                experimentalDecorators: false,
            };
            const ts = require("typescript");
            const tsconfigPath = props.tsconfigPath || ts_utils_1.findTsConfigFile(ts, context.config.context);
            if (tsconfigPath) {
                const host = ts_utils_1.createTypeScriptHost(ts);
                const json = ts_utils_1.readTsConfigFile(host, tsconfigPath);
                const tsconfig = ts_utils_1.parseJsonConfigFileContent(host, json);
                Object.assign(options, tsconfig.options);

                if (options.paths)
                    context.resolve.plugin(new tsconfig_paths_webpack_plugin_1.TsconfigPathsPlugin({
                        extensions: props.extensions,
                        configFile: tsconfigPath,
                    }));
            }

            custom.plugins.push(options.experimentalDecorators && ["@babel/plugin-proposal-decorators", { legacy: true }], ["@babel/plugin-proposal-class-properties", { loose: true }], constEnumPlugin(options.preserveConstEnums), inject_empty_exports_1.InjectEmptyExports);
        }

        custom.plugins = custom.plugins.filter(Boolean);
        const rule = Object.assign(Object.assign({ loader: require.resolve("./babel-custom-loader") }, lodash_1.omit(props, omitKeys)), { options: Object.assign({ custom }, props.options) });
        context.module.rule(rule);
        context.resolve.extension(...props.extensions);
    }
}
exports.BabelConfigurer = BabelConfigurer;
function extensionsToRegex(extensions) {
    if (extensions && extensions.length) {
        const group = extensions.map(x => x.slice(1)).join("|");
        const pattern = `\.(${group})$`;
        return new RegExp(pattern);
    }
}
function resolveDefaultExtensions(props) {
    const { useReact, useTypesScript } = props;
    return [
        ".js", ".es6", ".es", ".mjs",
        useReact && ".jsx",
        useTypesScript && [".ts", ".d.ts"],
        useTypesScript && useReact && ".tsx",
    ].filter(Boolean).flat();
}
function isReact(partial, plugins) {
    return someRequestName(partial.options.presets, isTypeScriptName)
        || someRequestName(partial.options.plugins, isTypeScriptName)
        || somePluginName(plugins, isTypeScriptName);
}
function isTypeScript(partial, plugins) {
    return someRequestName(partial.options.presets, isReactName)
        || someRequestName(partial.options.plugins, isReactName)
        || somePluginName(plugins, isReactName);
}

function isTypeScriptName(name) {
    return name.includes("typescript");
}

function isReactName(name) {
    return name.includes("react");
}

function constEnumPlugin(preserveConstEnums) {
    const transform = preserveConstEnums ? "removeConst" : "constObject";
    return ["babel-plugin-const-enum", { transform }];
}
function somePluginName(plugins, predicate) {
    if (plugins)
        for (const plugin of plugins)
            if (predicate(plugin.key))
                return true;
}
function someRequestName(items, predicate) {
    if (items)
        for (const item of items)
            if (predicate(requestOf(item)))
                return true;
}
function requestOf(item) {
    if (item) {
        if (typeof item === "string")
            return item;
        if (Array.isArray(item))
            return requestOf(item[0]);
        if (item.file)
            return item.file.request;
    }
}
//# sourceMappingURL=babel-configurer.js.map
